package com.daikin.wechat.api;

import com.daikin.wechat.config.ElasticsearchApi;
import com.daikin.wechat.entity.CustomAudience;
import com.daikin.wechat.entity.Targeting;
import com.google.gson.Gson;
import lombok.extern.log4j.Log4j2;
import org.apache.http.HttpHost;
import org.elasticsearch.action.DocWriteRequest;
import org.elasticsearch.action.DocWriteResponse;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.delete.DeleteResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.index.IndexResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.common.xcontent.XContentType;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.util.*;
@Log4j2
public class BulkApi {

    @Test
    public void bulkApiTest() {
        try (final RestHighLevelClient client = new RestHighLevelClient(
                RestClient.builder(
                        new HttpHost(ElasticsearchApi.HOST_NAME, ElasticsearchApi.PORT, ElasticsearchApi.SCHEME)
                )
        )) {
            final BulkRequest request = new BulkRequest()
                    .timeout(TimeValue.timeValueMinutes(2));
            final List<DocWriteRequest<?>> requests = new ArrayList();

            requests.add(new IndexRequest()
                    .index("test")
                    .source(get(), XContentType.JSON));
            requests.add(new IndexRequest()
                    .index("contact")
                    .source(get(), XContentType.JSON));
            request.add(requests
            );


            final BulkResponse bulkResponse = client.bulk(request, RequestOptions.DEFAULT);

            bulkResponse.forEach(bulkItemResponse -> {
                final DocWriteResponse response = bulkItemResponse.getResponse();

                System.out.println("response：\n" +
                        "index：" + response.getIndex() + "\n" +
                        "id：" + response.getId() + "\n" +
                        "version：" + response.getVersion());

                if (bulkItemResponse.getOpType() == DocWriteRequest.OpType.INDEX) {
                    final IndexResponse indexResponse = (IndexResponse) response;
                } else if (bulkItemResponse.getOpType() == DocWriteRequest.OpType.DELETE) {
                    final DeleteResponse deleteResponse = (DeleteResponse) response;
                }
            });
        } catch (final IOException e) {
            throw new RuntimeException(e);
        }
    }

    private String get(){

        CustomAudience customAudience = new CustomAudience();
        customAudience.setId("23842852893410737");
        customAudience.setName("类似受众人员 - ATC-intimate-180d");

        Targeting targeting = new Targeting();
        List<String> customAudiences = new ArrayList<>();
        customAudiences.add(customAudience.getId());
        targeting.setCustomAudiences(Arrays.asList(customAudience));

        targeting.setDevicePlatforms(Arrays.asList("mobile", "desktop"));

        Map<String, List<String>> geoLocations = new HashMap<>(1);
        geoLocations.put("location_types", Arrays.asList("home", "recent"));
        geoLocations.put("cities", Arrays.asList("beijing"));
        geoLocations.put("countries", Arrays.asList("国家"));
        targeting.setGeoLocations(geoLocations);

        targeting.setPublisherPlatforms(Arrays.asList("instagram", "messenger"));

//        log.info("{} 插入完成", JSONUtil.ObjectToJSONStringInSnakeCase(targeting));
        Gson gson = new Gson();

        return gson.toJson(targeting);
    }

}
